﻿using System;
using System.Collections;
using System.Collections.Generic;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// このサンプルは、ボタンとコンボボックスを配置したツールバーのサンプルです。
// ボタンをクリックすると、コンボボックスにある文字を、文字図形として
// 作図します。コンボボックスには、作図した文字の履歴が残ります。
// コマンドマネージャーに、コマンドを登録することも行っています。
//
// ※ ツールバーを新しくした場合は、一旦[ツール]メニューの[ユーザー設定]からツールバーを削除してください（必要なくなった場合も同様です）。
// Copyright @ 2014 RootPro Co.,Ltd.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace RCAddInToolBarCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        // GDI オブジェクトを削除するAPI（ツールバーのボタンに使うイメージのため）
        [System.Runtime.InteropServices.DllImport("gdi32.dll")]
        public static extern bool DeleteObject(IntPtr hObject);

        string toolBerName = "LineText";
        string buttonName1 = "RcSampleLineTextCommand1";// ボタンにセットする固有名
        string comboBoxName1 = "RcSampleLineTextcomboBox1";// コンボボックスにセットする固有名
        string commandResistName = "RcSampleLineTextCS";// コマンドマネージャー登録用コマンド名
        string commandName1 = "1 行文字列";// コマンド名
        int maxComboBoxListSize = 15;

        /// <summary>
        /// アドインの起動時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AppAddIn_Startup(object sender, EventArgs e)
        {
            // ツールバーのセット
            SetToolBer();
        }

        /// <summary>
        /// アドインの終了時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            // コマンドマネージャーに登録したコマンドに削除(終了時に必ず行うこと)
            CommandManager.RemoveCommand(commandResistName); // 追加したコマンド名
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion

        /// <summary>
        /// ツールバーのセット
        /// </summary>
        public void SetToolBer()
        {
            ToolBar[] lineTextToolbars = this.ToolBars.FindByText(toolBerName);
            ToolBar toolbar;

            // ツールバーがなかった場合
            if (lineTextToolbars == null)
            {
                // ツールバーの追加
                toolbar = this.ToolBars.Add(toolBerName);

                // ボタン1を追加
                ToolBarButtonItem button1 = (ToolBarButtonItem)toolbar.Items.Add(ToolBarItemType.Button);

                // ボタン1の表示をセット
                button1.DisplayStyle = ToolBarItemDisplayStyle.ImageAndText; // イメージ＆テキスト
                button1.Text = "文字";

                // 現在実行しているAssemblyを取得する
                System.Reflection.Assembly asm = this.GetType().Assembly;
                // ResourceManagerオブジェクトの作成
                string asssss = asm.GetName().Name;
                System.Resources.ResourceManager rm = new System.Resources.ResourceManager(asm.GetName().Name + ".Properties.Resources", asm);

                // ボタン1にイメージをセット
                System.Drawing.Bitmap bmp = (System.Drawing.Bitmap)rm.GetObject("LineTextCommandButtonImage");// 埋め込まれたリソース
                IntPtr handle = bmp.GetHbitmap();
                System.Drawing.Color color = System.Drawing.Color.FromArgb(255, 192, 192, 192);// ボタンのイメージに使っている背景色
                button1.SetHbitmap(handle, color.ToArgb());
                DeleteObject(handle); // イメージオブジェクトの削除（必ず行うこと）

                // ツールチップをセット
                button1.ToolTipText = "選ばれている文字を書きます。";

                // このボタンの固有名セット
                button1.Name = buttonName1;

                // クリックイベント時の関数登録
                button1.Click += new EventHandler(button_Click);

                // セパレータを追加
                toolbar.Items.Add(ToolBarItemType.Separator);


                // コンボボックスを追加
                ToolBarComboBoxItem comboBox = (ToolBarComboBoxItem)toolbar.Items.Add(ToolBarItemType.ComboBox);
                comboBox.DropDownStyle = ComboBoxStyle.DropDown;

                string[] str = { "ABC", "DEF", "GHI" };
                comboBox.SetItems(str);
                comboBox.SelectedIndex = 0; // 最初の文字を選択

                comboBox.ToolTipText = "文字列を入力するか選びます。";

                // このコンボボックスの固有名セット
                comboBox.Name = comboBoxName1;
            }
            // ツールバーがすでににある場合
            else
            {
                toolbar = lineTextToolbars[0];

                // ボタン1がある場合
                ToolBarItem[] toolBarButtonItem = toolbar.Items.Find(buttonName1);
                if (toolBarButtonItem.Length != 0)
                {
                    ToolBarButtonItem button1 = (ToolBarButtonItem)toolBarButtonItem[0];

                    // クリックイベント時の関数登録
                    button1.Click += new EventHandler(button_Click);
                }

            }


            // この"文字"コマンドをコマンドマネージャーに登録 // (コマンドウィンドウで操作するコマンドになります)
            Command command = CommandManager.AddCommand(commandResistName);　// コマンドマネージャー登録用コマンド名
            command.Name = commandName1;// コマンド名（実行時にコマンドウィンドウに表示されます）
            // 座標パラメータの追加
            CommandParameterGroupCollection parameterGroups = command.ParameterGroups;
            CommandParameterGroup parameterGroup = parameterGroups.Add(ParameterItemType.Point, 0, "配置点座標");

            // パラメータの変更イベント登録
            command.ParameterChanged += new CommandParameterChangedEventHandler(command_ParameterChanged); // イベント処理関数名:command_ParameterChanged 
        }


        /// <summary>
        /// ボタン1がクリックされたときにイベント処理(文字を書く)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void button_Click(object sender, EventArgs e)
        {
            // コマンド実行(コマンドマネージャー登録用コマンド名を使う)
            CommandManager.ExecuteCommand(commandResistName);
        }

        /// <summary>
        /// コマンドマネージャーのパラメータ変更イベント処理 //（配置点座標が入力されたとき）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_ParameterChanged(Object sender, CommandParameterChangedEventArgs e)
        {
            Command command = (Command)sender;
            int id = e.ParameterID;// 変更のあったパラメータID
            ParameterItem parameterItem = command.GetParameterItemByID(id); // パラメータアイテム

            // パラメータアイテムが座標タイプの場合
            if (parameterItem.ParameterType == ParameterItemType.Point)
            {
                PointParameterItem pointParameterItem = (PointParameterItem)parameterItem;
                Point2d point = pointParameterItem.Point; // 配置点座標の取得

                // 表の作成を実行する
                DrawLineText(point);
            }

            e.CommandState = CommandState.End; // 終了
        }

        /// <summary>
        /// コンボボックスに書かれている文字を書く
        /// </summary>
        /// <param name="locatePoint">配置点座標</param>
        private void DrawLineText(Point2d locatePoint)
        {
            // ツールバーを探す
            ToolBar[] lineTextToolbars = this.ToolBars.FindByText(toolBerName);
            if (lineTextToolbars == null)
                return;

            ToolBar toolbar = lineTextToolbars[0];

            // コンボボックスがある場合
            ToolBarItem[] toolBarComboBoxItem = toolbar.Items.Find(comboBoxName1);
            if (toolBarComboBoxItem.Length != 0)
            {
                ToolBarComboBoxItem comboBox1 = (ToolBarComboBoxItem)toolBarComboBoxItem[0];

                // 文字列取得
                string comboBoxText = comboBox1.Text;
                if (comboBoxText != "")
                {
                    Document doc = ActiveDocument;          // アクティブな図面
                    Drawing drawing = doc.CurrentDrawing;   // カレント部分図

                    TextShape textShape = drawing.Shapes.AddText(comboBoxText, locatePoint, 0);
                }

                // コンボボックスのリスト更新
                int selectedIndex = comboBox1.SelectedIndex;
                string[] comboBoxTexts = comboBox1.GetItems();
                int listSize = comboBoxTexts.Length;
                int sameTextIndex = -1;
                for (int i = listSize - 1; i >= 0; i--)
                {
                    string str1 = comboBoxTexts[i];
                    // 同じ文字列があるかチェック
                    if (comboBoxText == str1)
                    {
                        sameTextIndex = i;
                    }
                }


                int newListSize = listSize;
                // 同じ文字がなかった場合
                if (sameTextIndex < 0)
                    newListSize = Math.Min(listSize + 1, maxComboBoxListSize);

                string[] newComboBoxTexts = new string[newListSize];
                for (int i = 0; i < newListSize; i++)
                {
                    // 同じ文字がなかった場合は先頭に追加
                    if (sameTextIndex < 0)
                    {
                        if (i == 0)
                            newComboBoxTexts[i] = comboBoxText;
                        else
                            newComboBoxTexts[i] = comboBoxTexts[i - 1];
                    }
                    // 同じ文字があった場合は先頭に移動
                    else
                    {
                        if (i == 0)
                            newComboBoxTexts[i] = comboBoxText;
                        else
                        {
                            // 同じ文字が先頭だった場合
                            if (sameTextIndex == 0)
                            {
                                newComboBoxTexts[i] = comboBoxTexts[i];
                            }
                            else
                            {
                                if (i <= sameTextIndex)
                                    newComboBoxTexts[i] = comboBoxTexts[i - 1];
                                else
                                    newComboBoxTexts[i] = comboBoxTexts[i];
                            }

                        }
                    }
                }

                comboBox1.SetItems(newComboBoxTexts);
                comboBox1.SelectedIndex = 0;
            }
        }
    }
}
